@@ -11,12 +11,23 @@ from django.core.cache import cache
 from django.db import connection
 from django.http import HttpResponse
 from django.template.loader import get_template
-from django.test import AsyncRequestFactory, RequestFactory
+from django.test import RequestFactory
 from django.test.utils import override_settings
 
 from debug_toolbar.forms import SignedDataForm
-from debug_toolbar.middleware import DebugToolbarMiddleware, show_toolbar
+from debug_toolbar.middleware import (
+    DebugToolbarMiddleware,
+    show_toolbar,
+    show_toolbar_with_docker,
+)
 from debug_toolbar.panels import Panel
+from debug_toolbar.panels.cache import CachePanel
+from debug_toolbar.panels.history import HistoryPanel
+from debug_toolbar.panels.request import RequestPanel
+from debug_toolbar.panels.sql import SQLPanel
+from debug_toolbar.panels.templates import TemplatesPanel
+from debug_toolbar.panels.versions import VersionsPanel
+from debug_toolbar.store import get_store
 from debug_toolbar.toolbar import DebugToolbar
 
 from .base import BaseTestCase, IntegrationTestCase
@@ -36,13 +47,13 @@ except ImportError:
 rf = RequestFactory()
 
 
-def toolbar_store_id():
+def toolbar_request_id():
     def get_response(request):
         return HttpResponse()
 
     toolbar = DebugToolbar(rf.get("/"), get_response)
-    toolbar.store()
-    return toolbar.store_id
+    toolbar.init_store()
+    return toolbar.request_id
 
 
 class BuggyPanel(Panel):
@@ -72,7 +83,8 @@ class DebugToolbarTestCase(BaseTestCase):
         with self.settings(INTERNAL_IPS=[]):
             # Is true because REMOTE_ADDR is 127.0.0.1 and the 255
             # is shifted to be 1.
-            self.assertTrue(show_toolbar(self.request))
+            self.assertFalse(show_toolbar(self.request))
+            self.assertTrue(show_toolbar_with_docker(self.request))
         mocked_gethostbyname.assert_called_once_with("host.docker.internal")
 
     def test_not_iterating_over_INTERNAL_IPS(self):
@@ -107,45 +119,12 @@ class DebugToolbarTestCase(BaseTestCase):
         toolbar.config["RENDER_PANELS"] = True
         self.assertTrue(toolbar.should_render_panels())
         toolbar.config["RENDER_PANELS"] = None
-        self.assertTrue(toolbar.should_render_panels())
-
-    def test_should_render_panels_multiprocess(self):
-        """
-        The toolbar should render the panels on each request when wsgi.multiprocess
-        is True or missing.
-        """
-        request = rf.get("/")
-        request.META["wsgi.multiprocess"] = True
-        toolbar = DebugToolbar(request, self.get_response)
-        toolbar.config["RENDER_PANELS"] = None
-        self.assertTrue(toolbar.should_render_panels())
-
-        request.META["wsgi.multiprocess"] = False
-        self.assertFalse(toolbar.should_render_panels())
-
-        request.META.pop("wsgi.multiprocess")
-        self.assertTrue(toolbar.should_render_panels())
-
-    def test_should_render_panels_asgi(self):
-        """
-        The toolbar not should render the panels on each request when wsgi.multiprocess
-        is True or missing in case of async context rather than multithreaded
-        wsgi.
-        """
-        async_request = AsyncRequestFactory().get("/")
-        # by default ASGIRequest will have wsgi.multiprocess set to True
-        # but we are still assigning this to true cause this could change
-        # and we specifically need to check that method returns false even with
-        # wsgi.multiprocess set to true
-        async_request.META["wsgi.multiprocess"] = True
-        toolbar = DebugToolbar(async_request, self.get_response)
-        toolbar.config["RENDER_PANELS"] = None
         self.assertFalse(toolbar.should_render_panels())
 
     def _resolve_stats(self, path):
         # takes stats from Request panel
         request = rf.get(path)
-        panel = self.toolbar.get_panel_by_id("RequestPanel")
+        panel = self.toolbar.get_panel_by_id(RequestPanel.panel_id)
         response = panel.process_request(request)
         panel.generate_stats(request, response)
         return panel.get_stats()
@@ -196,9 +175,13 @@ class DebugToolbarTestCase(BaseTestCase):
         # may run earlier and cause fewer cache calls.
         cache.clear()
         response = self.client.get("/cached_view/")
-        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 3)
+        self.assertEqual(
+            len(response.toolbar.get_panel_by_id(CachePanel.panel_id).calls), 3
+        )
         response = self.client.get("/cached_view/")
-        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 2)
+        self.assertEqual(
+            len(response.toolbar.get_panel_by_id(CachePanel.panel_id).calls), 2
+        )
 
     @override_settings(ROOT_URLCONF="tests.urls_use_package_urls")
     def test_include_package_urls(self):
@@ -207,16 +190,24 @@ class DebugToolbarTestCase(BaseTestCase):
         # may run earlier and cause fewer cache calls.
         cache.clear()
         response = self.client.get("/cached_view/")
-        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 3)
+        self.assertEqual(
+            len(response.toolbar.get_panel_by_id(CachePanel.panel_id).calls), 3
+        )
         response = self.client.get("/cached_view/")
-        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 2)
+        self.assertEqual(
+            len(response.toolbar.get_panel_by_id(CachePanel.panel_id).calls), 2
+        )
 
     def test_low_level_cache_view(self):
         """Test cases when low level caching API is used within a request."""
         response = self.client.get("/cached_low_level_view/")
-        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 2)
+        self.assertEqual(
+            len(response.toolbar.get_panel_by_id(CachePanel.panel_id).calls), 2
+        )
         response = self.client.get("/cached_low_level_view/")
-        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 1)
+        self.assertEqual(
+            len(response.toolbar.get_panel_by_id(CachePanel.panel_id).calls), 1
+        )
 
     def test_cache_disable_instrumentation(self):
         """
@@ -228,7 +219,9 @@ class DebugToolbarTestCase(BaseTestCase):
         response = self.client.get("/execute_sql/")
         self.assertEqual(cache.get("UseCacheAfterToolbar.before"), 1)
         self.assertEqual(cache.get("UseCacheAfterToolbar.after"), 1)
-        self.assertEqual(len(response.toolbar.get_panel_by_id("CachePanel").calls), 0)
+        self.assertEqual(
+            len(response.toolbar.get_panel_by_id(CachePanel.panel_id).calls), 0
+        )
 
     def test_is_toolbar_request(self):
         request = rf.get("/__debug__/render_panel/")
@@ -275,7 +268,7 @@ class DebugToolbarTestCase(BaseTestCase):
 
     def test_data_gone(self):
         response = self.client.get(
-            "/__debug__/render_panel/?store_id=GONE&panel_id=RequestPanel"
+            "/__debug__/render_panel/?request_id=GONE&panel_id=RequestPanel"
         )
         self.assertIn("Please reload the page and retry.", response.json()["content"])
 
@@ -330,9 +323,13 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
             raise self.failureException(msg)
 
     def test_render_panel_checks_show_toolbar(self):
-        url = "/__debug__/render_panel/"
-        data = {"store_id": toolbar_store_id(), "panel_id": "VersionsPanel"}
+        request_id = toolbar_request_id()
+        get_store().save_panel(
+            request_id, VersionsPanel.panel_id, {"value": "Test data"}
+        )
+        data = {"request_id": request_id, "panel_id": VersionsPanel.panel_id}
 
+        url = "/__debug__/render_panel/"
         response = self.client.get(url, data)
         self.assertEqual(response.status_code, 200)
         response = self.client.get(
@@ -349,18 +346,20 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
 
     def test_middleware_render_toolbar_json(self):
         """Verify the toolbar is rendered and data is stored for a json request."""
-        self.assertEqual(len(DebugToolbar._store), 0)
+        store = get_store()
+        self.assertEqual(len(list(store.request_ids())), 0)
 
         data = {"foo": "bar"}
         response = self.client.get("/json_view/", data, content_type="application/json")
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.content.decode("utf-8"), '{"foo": "bar"}')
         # Check the history panel's stats to verify the toolbar rendered properly.
-        self.assertEqual(len(DebugToolbar._store), 1)
-        toolbar = list(DebugToolbar._store.values())[0]
+        request_ids = list(store.request_ids())
+        self.assertEqual(len(request_ids), 1)
+        toolbar = DebugToolbar.fetch(request_ids[0])
         self.assertEqual(
-            toolbar.get_panel_by_id("HistoryPanel").get_stats()["data"],
-            {"foo": ["bar"]},
+            toolbar.get_panel_by_id(HistoryPanel.panel_id).get_stats()["data"],
+            {"foo": "bar"},
         )
 
     def test_template_source_checks_show_toolbar(self):
@@ -406,15 +405,19 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
         self.assertContains(response, "Template Does Not Exist: does_not_exist.html")
 
     def test_sql_select_checks_show_toolbar(self):
+        self.client.get("/execute_sql/")
+        request_ids = list(get_store().request_ids())
+        request_id = request_ids[-1]
+        toolbar = DebugToolbar.fetch(request_id, SQLPanel.panel_id)
+        panel = toolbar.get_panel_by_id(SQLPanel.panel_id)
+        djdt_query_id = panel.get_stats()["queries"][-1]["djdt_query_id"]
+
         url = "/__debug__/sql_select/"
         data = {
             "signed": SignedDataForm.sign(
                 {
-                    "sql": "SELECT * FROM auth_user",
-                    "raw_sql": "SELECT * FROM auth_user",
-                    "params": "{}",
-                    "alias": "default",
-                    "duration": "0",
+                    "request_id": request_id,
+                    "djdt_query_id": djdt_query_id,
                 }
             )
         }
@@ -434,15 +437,19 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
             self.assertEqual(response.status_code, 404)
 
     def test_sql_explain_checks_show_toolbar(self):
+        self.client.get("/execute_sql/")
+        request_ids = list(get_store().request_ids())
+        request_id = request_ids[-1]
+        toolbar = DebugToolbar.fetch(request_id, SQLPanel.panel_id)
+        panel = toolbar.get_panel_by_id(SQLPanel.panel_id)
+        djdt_query_id = panel.get_stats()["queries"][-1]["djdt_query_id"]
+
         url = "/__debug__/sql_explain/"
         data = {
             "signed": SignedDataForm.sign(
                 {
-                    "sql": "SELECT * FROM auth_user",
-                    "raw_sql": "SELECT * FROM auth_user",
-                    "params": "{}",
-                    "alias": "default",
-                    "duration": "0",
+                    "request_id": request_id,
+                    "djdt_query_id": djdt_query_id,
                 }
             )
         }
@@ -468,15 +475,19 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
         """
         Confirm select queries that start with a parenthesis can be explained.
         """
+        self.client.get("/execute_union_sql/")
+        request_ids = list(get_store().request_ids())
+        request_id = request_ids[-1]
+        toolbar = DebugToolbar.fetch(request_id, SQLPanel.panel_id)
+        panel = toolbar.get_panel_by_id(SQLPanel.panel_id)
+        djdt_query_id = panel.get_stats()["queries"][-1]["djdt_query_id"]
+
         url = "/__debug__/sql_explain/"
         data = {
             "signed": SignedDataForm.sign(
                 {
-                    "sql": "(SELECT * FROM auth_user) UNION (SELECT * from auth_user)",
-                    "raw_sql": "(SELECT * FROM auth_user) UNION (SELECT * from auth_user)",
-                    "params": "{}",
-                    "alias": "default",
-                    "duration": "0",
+                    "request_id": request_id,
+                    "djdt_query_id": djdt_query_id,
                 }
             )
         }
@@ -488,19 +499,19 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
         connection.vendor == "postgresql", "Test valid only on PostgreSQL"
     )
     def test_sql_explain_postgres_json_field(self):
+        self.client.get("/execute_json_sql/")
+        request_ids = list(get_store().request_ids())
+        request_id = request_ids[-1]
+        toolbar = DebugToolbar.fetch(request_id, SQLPanel.panel_id)
+        panel = toolbar.get_panel_by_id(SQLPanel.panel_id)
+        djdt_query_id = panel.get_stats()["queries"][-1]["djdt_query_id"]
+
         url = "/__debug__/sql_explain/"
-        base_query = (
-            'SELECT * FROM "tests_postgresjson" WHERE "tests_postgresjson"."field" @>'
-        )
-        query = base_query + """ '{"foo": "bar"}'"""
         data = {
             "signed": SignedDataForm.sign(
                 {
-                    "sql": query,
-                    "raw_sql": base_query + " %s",
-                    "params": '["{\\"foo\\": \\"bar\\"}"]',
-                    "alias": "default",
-                    "duration": "0",
+                    "request_id": request_id,
+                    "djdt_query_id": djdt_query_id,
                 }
             )
         }
@@ -519,15 +530,19 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
             self.assertEqual(response.status_code, 404)
 
     def test_sql_profile_checks_show_toolbar(self):
+        self.client.get("/execute_sql/")
+        request_ids = list(get_store().request_ids())
+        request_id = request_ids[-1]
+        toolbar = DebugToolbar.fetch(request_id, SQLPanel.panel_id)
+        panel = toolbar.get_panel_by_id(SQLPanel.panel_id)
+        djdt_query_id = panel.get_stats()["queries"][-1]["djdt_query_id"]
+
         url = "/__debug__/sql_profile/"
         data = {
             "signed": SignedDataForm.sign(
                 {
-                    "sql": "SELECT * FROM auth_user",
-                    "raw_sql": "SELECT * FROM auth_user",
-                    "params": "{}",
-                    "alias": "default",
-                    "duration": "0",
+                    "request_id": request_id,
+                    "djdt_query_id": djdt_query_id,
                 }
             )
         }
@@ -556,7 +571,7 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
         response = self.client.get(url)
         self.assertIn(b'id="djDebug"', response.content)
         # Verify the store id is not included.
-        self.assertNotIn(b"data-store-id", response.content)
+        self.assertNotIn(b"data-request-id", response.content)
         # Verify the history panel was disabled
         self.assertIn(
             b'<input type="checkbox" data-cookie="djdtHistoryPanel" '
@@ -576,7 +591,7 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
         response = self.client.get(url)
         self.assertIn(b'id="djDebug"', response.content)
         # Verify the store id is included.
-        self.assertIn(b"data-store-id", response.content)
+        self.assertIn(b"data-request-id", response.content)
         # Verify the history panel was not disabled
         self.assertNotIn(
             b'<input type="checkbox" data-cookie="djdtHistoryPanel" '
@@ -626,10 +641,10 @@ class DebugToolbarIntegrationTestCase(IntegrationTestCase):
         parser = html5lib.HTMLParser()
         doc = parser.parse(response.content)
         el = doc.find(".//*[@id='djDebug']")
-        store_id = el.attrib["data-store-id"]
+        request_id = el.attrib["data-request-id"]
         response = self.client.get(
             "/__debug__/render_panel/",
-            {"store_id": store_id, "panel_id": "TemplatesPanel"},
+            {"request_id": request_id, "panel_id": TemplatesPanel.panel_id},
         )
         self.assertEqual(response.status_code, 200)
         # The key None (without quotes) exists in the list of template
@@ -649,6 +664,9 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
         options = Options()
         if os.environ.get("CI"):
             options.add_argument("-headless")
+        # Set the browser preference to light mode for consistent testing
+        options.set_preference("ui.systemUsesDarkTheme", 0)
+        options.set_preference("ui.prefersReducedMotion", 0)
         cls.selenium = webdriver.Firefox(options=options)
 
     @classmethod
@@ -665,14 +683,14 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
 
     def test_basic(self):
         self.get("/regular/basic/")
-        version_panel = self.selenium.find_element(By.ID, "VersionsPanel")
+        version_panel = self.selenium.find_element(By.ID, VersionsPanel.panel_id)
 
         # Versions panel isn't loaded
         with self.assertRaises(NoSuchElementException):
             version_panel.find_element(By.TAG_NAME, "table")
 
         # Click to show the versions panel
-        self.selenium.find_element(By.CLASS_NAME, "VersionsPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, VersionsPanel.panel_id).click()
 
         # Version panel loads
         table = self.wait.until(
@@ -688,15 +706,15 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
     )
     def test_basic_jinja(self):
         self.get("/regular_jinja/basic")
-        template_panel = self.selenium.find_element(By.ID, "TemplatesPanel")
+        template_panel = self.selenium.find_element(By.ID, TemplatesPanel.panel_id)
 
         # Click to show the template panel
-        self.selenium.find_element(By.CLASS_NAME, "TemplatesPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, TemplatesPanel.panel_id).click()
+
         # This should be 2 templates rendered, including base.html See
         # JinjaTemplateTestCase.test_django_jinja2_parent_template_instrumented
         self.assertIn("Templates (1 rendered)", template_panel.text)
-        self.assertNotIn("base.html", template_panel.text)
-        self.assertIn("jinja2/basic.jinja", template_panel.text)
+        self.assertIn("basic.jinja", template_panel.text)
 
     @override_settings(
         DEBUG_TOOLBAR_CONFIG={
@@ -707,14 +725,14 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
         self.get("/regular_jinja/basic")
         # Make a new request so the history panel has more than one option.
         self.get("/execute_sql/")
-        template_panel = self.selenium.find_element(By.ID, "HistoryPanel")
+        template_panel = self.selenium.find_element(By.ID, HistoryPanel.panel_id)
         # Record the current side panel of buttons for later comparison.
         previous_button_panel = self.selenium.find_element(
             By.ID, "djDebugPanelList"
         ).text
 
         # Click to show the history panel
-        self.selenium.find_element(By.CLASS_NAME, "HistoryPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, HistoryPanel.panel_id).click()
         # Click to switch back to the jinja page view snapshot
         list(template_panel.find_elements(By.CSS_SELECTOR, "button"))[-1].click()
 
@@ -729,10 +747,10 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
     @override_settings(DEBUG_TOOLBAR_CONFIG={"RESULTS_CACHE_SIZE": 0})
     def test_expired_store(self):
         self.get("/regular/basic/")
-        version_panel = self.selenium.find_element(By.ID, "VersionsPanel")
+        version_panel = self.selenium.find_element(By.ID, VersionsPanel.panel_id)
 
         # Click to show the version panel
-        self.selenium.find_element(By.CLASS_NAME, "VersionsPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, VersionsPanel.panel_id).click()
 
         # Version panel doesn't loads
         error = self.wait.until(
@@ -760,10 +778,10 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
     )
     def test_django_cached_template_loader(self):
         self.get("/regular/basic/")
-        version_panel = self.selenium.find_element(By.ID, "TemplatesPanel")
+        version_panel = self.selenium.find_element(By.ID, TemplatesPanel.panel_id)
 
         # Click to show the templates panel
-        self.selenium.find_element(By.CLASS_NAME, "TemplatesPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, TemplatesPanel.panel_id).click()
 
         # Templates panel loads
         trigger = self.wait.until(
@@ -780,11 +798,11 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
 
     def test_sql_action_and_go_back(self):
         self.get("/execute_sql/")
-        sql_panel = self.selenium.find_element(By.ID, "SQLPanel")
+        sql_panel = self.selenium.find_element(By.ID, SQLPanel.panel_id)
         debug_window = self.selenium.find_element(By.ID, "djDebugWindow")
 
         # Click to show the SQL panel
-        self.selenium.find_element(By.CLASS_NAME, "SQLPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, SQLPanel.panel_id).click()
 
         # SQL panel loads
         button = self.wait.until(
@@ -807,9 +825,9 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
     def test_displays_server_error(self):
         self.get("/regular/basic/")
         debug_window = self.selenium.find_element(By.ID, "djDebugWindow")
-        self.selenium.find_element(By.CLASS_NAME, "BuggyPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, BuggyPanel.panel_id).click()
         self.wait.until(EC.visibility_of(debug_window))
-        self.assertEqual(debug_window.text, "»\n500: Internal Server Error")
+        self.assertEqual(debug_window.text, "500: Internal Server Error\n»")
 
     def test_toolbar_language_will_render_to_default_language_when_not_set(self):
         self.get("/regular/basic/")
@@ -817,10 +835,10 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
         assert hide_button.text == "Hide »"
 
         self.get("/execute_sql/")
-        sql_panel = self.selenium.find_element(By.ID, "SQLPanel")
+        sql_panel = self.selenium.find_element(By.ID, SQLPanel.panel_id)
 
         # Click to show the SQL panel
-        self.selenium.find_element(By.CLASS_NAME, "SQLPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, SQLPanel.panel_id).click()
 
         table = self.wait.until(
             lambda selenium: sql_panel.find_element(By.TAG_NAME, "table")
@@ -835,10 +853,10 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
         assert hide_button.text == "Esconder »"
 
         self.get("/execute_sql/")
-        sql_panel = self.selenium.find_element(By.ID, "SQLPanel")
+        sql_panel = self.selenium.find_element(By.ID, SQLPanel.panel_id)
 
         # Click to show the SQL panel
-        self.selenium.find_element(By.CLASS_NAME, "SQLPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, SQLPanel.panel_id).click()
 
         table = self.wait.until(
             lambda selenium: sql_panel.find_element(By.TAG_NAME, "table")
@@ -854,10 +872,10 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
         assert hide_button.text == "Hide »"
 
         self.get("/execute_sql/")
-        sql_panel = self.selenium.find_element(By.ID, "SQLPanel")
+        sql_panel = self.selenium.find_element(By.ID, SQLPanel.panel_id)
 
         # Click to show the SQL panel
-        self.selenium.find_element(By.CLASS_NAME, "SQLPanel").click()
+        self.selenium.find_element(By.CLASS_NAME, SQLPanel.panel_id).click()
 
         table = self.wait.until(
             lambda selenium: sql_panel.find_element(By.TAG_NAME, "table")
@@ -878,11 +896,17 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
         self.get("/ajax/")
         make_ajax = self.selenium.find_element(By.ID, "click_for_ajax")
         make_ajax.click()
+        # Sleep a tad to avoid a selenium.common.exceptions.StaleElementReferenceException
+        # when looking for the small text of the history panel
+        time.sleep(0.1)
         # Need to wait until the ajax request is over and json_view is displayed on the toolbar
-        time.sleep(2)
-        history_panel = self.wait.until(
-            lambda selenium: self.selenium.find_element(By.ID, "djdt-HistoryPanel")
+        self.wait.until(
+            lambda selenium: self.selenium.find_element(
+                By.CSS_SELECTOR, "#djdt-HistoryPanel a.HistoryPanel small"
+            ).text
+            == "/json_view/"
         )
+        history_panel = self.selenium.find_element(By.ID, "djdt-HistoryPanel")
         self.assertNotIn("/ajax/", history_panel.text)
         self.assertIn("/json_view/", history_panel.text)
 
@@ -892,26 +916,35 @@ class DebugToolbarLiveTestCase(StaticLiveServerTestCase):
         toolbar = self.selenium.find_element(By.ID, "djDebug")
 
         # Check that the default theme is auto
-        self.assertEqual(toolbar.get_attribute("data-theme"), "auto")
+        self.assertEqual(toolbar.get_attribute("data-user-theme"), "auto")
 
         # The theme toggle button is shown on the toolbar
         toggle_button = self.selenium.find_element(By.ID, "djToggleThemeButton")
         self.assertTrue(toggle_button.is_displayed())
 
-        # The theme changes when user clicks the button
-        toggle_button.click()
+        # The browser is set to light mode via Firefox preferences
+        # With light mode system preference, the order is: auto -> dark -> light -> auto
+        # Check that auto initially uses light theme
+        self.assertEqual(toolbar.get_attribute("data-user-theme"), "auto")
         self.assertEqual(toolbar.get_attribute("data-theme"), "light")
-        toggle_button.click()
+
+        # The theme changes when user clicks the button
+        toggle_button.click()  # auto -> dark
+        self.assertEqual(toolbar.get_attribute("data-user-theme"), "dark")
         self.assertEqual(toolbar.get_attribute("data-theme"), "dark")
-        toggle_button.click()
-        self.assertEqual(toolbar.get_attribute("data-theme"), "auto")
-        # Switch back to light.
-        toggle_button.click()
+
+        toggle_button.click()  # dark -> light
+        self.assertEqual(toolbar.get_attribute("data-user-theme"), "light")
+        self.assertEqual(toolbar.get_attribute("data-theme"), "light")
+
+        toggle_button.click()  # light -> auto
+        self.assertEqual(toolbar.get_attribute("data-user-theme"), "auto")
         self.assertEqual(toolbar.get_attribute("data-theme"), "light")
 
         # Enter the page again to check that user settings is saved
         self.get("/regular/basic/")
         toolbar = self.selenium.find_element(By.ID, "djDebug")
+        self.assertEqual(toolbar.get_attribute("data-user-theme"), "auto")
         self.assertEqual(toolbar.get_attribute("data-theme"), "light")
 
     def test_async_sql_action(self):
